import meshio
import numpy as np
from scipy.stats import lognorm

# Load the .vtu mesh file
mesh = meshio.read("domain.vtu")

# Print the available cell block types
print("Available cell block types:")
for cell_block in mesh.cells:
    print(cell_block.type)

# Assuming the desired cell block type is "quad" (i.e., quadrilateral element)
cell_block_type = "quad"

# Get the cell indices for the quad block
quad_indices = None
for cell_block in mesh.cells:
    if cell_block.type == cell_block_type:
        quad_indices = cell_block.data
        break

if quad_indices is None:
    raise ValueError("Quad cell block not found")

# Define the minimum and maximum permeability values
min_perm = 1e-16
max_perm = 1e-12

# Define the mean and standard deviation of the log-normal distribution
mean_log_perm = np.log(min_perm)
stddev_log_perm = (np.log(max_perm) - np.log(min_perm)) / 3.0

# Generate log-normal permeability values within the specified range for each node
num_nodes = len(mesh.points)
permeability_values = np.zeros((num_nodes, 4))

for i in range(num_nodes):
    # Generate log-normal permeability values for the 2D approximation
    log_kxx = lognorm.rvs(stddev_log_perm, scale=np.exp(mean_log_perm))
    log_kzz = lognorm.rvs(stddev_log_perm, scale=np.exp(mean_log_perm))

    # Clip the permeability values to ensure they are within the specified range
    kxx = np.clip(log_kxx, min_perm, max_perm)
    kyy = 0.0
    kxy = 0.0
    kzz = np.clip(log_kzz, min_perm, max_perm)
    

    # Assign the permeability values to the 2D approximation (4x1 voigt notation vector)
    permeability_values[i] = [kxx, kyy, kxy, kzz]


# Assign the point data to the mesh
mesh.cell_data['permeability_tensor'] = [np.array(permeability_values)]

# Save the updated mesh with permeability values to a new .vtu file
meshio.write("revised_domain.vtu", mesh)
